# ----------------------------------------------------------------------
# Curses::UI::Language::russian
# This file is in KOI8-R encoding
# Maintainer: maurice@gitaar.net, cursesui@nslu.x.myxomop.com
# ----------------------------------------------------------------------

package Curses::UI::Language::russian; 
1;
__DATA__

# ----------------------------------------------------------------------
# For Curses::UI
# ----------------------------------------------------------------------

[screen_too_small]
Размер терминала не достаточен для этого приложения.
Увеличте размер терминала и перезапустите программу.
Нажмите ^C для выхода. 

# ----------------------------------------------------------------------
# For Curses::UI::Calendar
# ----------------------------------------------------------------------

[months]
Январь Февраль Март Апрель Май Июнь Июль
Август Сентябрь Октябрь Ноябрь Декабрь

[days_short] Вс Пн Вт Ср Чт Пт Сб

# ----------------------------------------------------------------------
# For Curses::UI::Buttonbox
# ----------------------------------------------------------------------

[button_ok]      o:OK
[button_cancel]  c:Отменить
[button_yes]     y:Да
[button_no]      n:Нет

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::Error
# ----------------------------------------------------------------------

[error_title]    Сообщение об ошибке

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::FileBrowser
# ----------------------------------------------------------------------

# The filebrowser title
[file_title]     Выберите фаил
[file_savetitle] Выберите файл для сохранения
[file_loadtitle] Выберите файл для загрузки

# The labels for the dialog screen.
[file_path]      Путь :
[file_file]      Фаил :
[file_mask]      Фильтр :

# The size of the longest label
[file_labelsize] 6

# On directory up
[file_dirup] Предыдущая директория

# For asking the user if a file may be overwritten
[file_overwrite_title] Подтверждение
[file_overwrite_question_pre] 
Вы действительно хотете записать поверх файла "
[file_overwrite_question_post] 
"?

# Errors
[file_err_opendir_pre] Ошибка при открытии директории "
[file_err_opendir_post] "
[file_err_nofileselected] Вы не выбрали фаил!


