# ----------------------------------------------------------------------
# Curses::UI::Language::german
# Maintainer: kleine@ak.sax.de
# ----------------------------------------------------------------------

package Curses::UI::Language::german;
1;
__DATA__

# ----------------------------------------------------------------------
# For Curses::UI
# ----------------------------------------------------------------------

[screen_too_small]
Die Fenstergröße ist zu klein für dieses Programm.
Bitte Größe Ändern und das Programm neu starten.
<STRG+C> drücken zum Beenden...

# ----------------------------------------------------------------------
# For Curses::UI::Calendar
# ----------------------------------------------------------------------

[months]
Januar Februar März April Mai Juni Juli
August September Oktober November Dezember

[days_short] Su Mo Di Mi Do Fr Sa

# ----------------------------------------------------------------------
# For Curses::UI::Buttonbox
# ----------------------------------------------------------------------

[button_ok]      o:OK
[button_cancel]  a:Abbrechen
[button_yes]     j:Ja
[button_no]      n:Nein

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::Error
# ----------------------------------------------------------------------

[error_title]    Fehlermeldung

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::FileBrowser
# ----------------------------------------------------------------------

# The filebrowser title
[file_title]     Dateiauswahl
[file_savetitle] Dateiauswahl zum Speichern
[file_loadtitle] Dateiauswahl zum Laden

# The dirbrowser title
[dir_title]     Verzeichnisauswahl

# The labels for the dialog screen.
[file_path]      Pfad  :
[file_file]      Datei :
[file_mask]      Maske :

# The size of the longest label
[file_labelsize] 7

# On directory up
[file_dirup] Höheres Verzeichnis

# For asking the user if a file may be overwritten
[file_overwrite_title] Bestätigung
[file_overwrite_question_pre]
Datei wirklich Überschreiben "
[file_overwrite_question_post]
" ?

# Errors
[file_err_opendir_pre] Kann Verzeichnis nicht öffnen "
[file_err_opendir_post] "
[file_err_nofileselected] Keine Datei ausgewählt!



